/****************************************************************************
 *
 * Project: LPC-2478-STK
 *
 * Copyright: Ivan Vasilev, Olimex Ltd. All rights reserved.
 *
 * File: exttest.c
 * Description: extension test
 * Developer: Ivan Vasilev, <ivan at l123.org>
 *
 * Last change: $Date: 2008-07-01 14:12:20 +0300 (???????, 01 ??? 2008) $
 * Revision: $Revision: 58 $
 * Id: $Id: iBMapp.c 58 2008-07-01 11:12:20Z Ivan $
 * Author: $Author: Ivan $
 *
 ****************************************************************************/

#include "exttest.h"
#include "string.h"
#include "stdio.h"

test_ext_pin_t pinset[NUMBER_TESTED_PINS];

test_ext_pin_t pull_pin;

test_ext_pin_t led_pin;


void PinsetInit(void)
{
	memcpy((void *)&pinset[0].pin_name, (const char *)&"P0_02", PIN_NAME_SIZE);
	pinset[0].pin_offset = 1<<2;
	pinset[0].pin_get = &FIO0PIN;
	pinset[0].pin_set = &FIO0SET;
	pinset[0].pin_clr = &FIO0CLR;
	pinset[0].pin_dir = &FIO0DIR;
	
	memcpy((void *)&pinset[1].pin_name, (const char *)&"P0_03", PIN_NAME_SIZE);
	pinset[1].pin_offset = 1<<3;
	pinset[1].pin_get = &FIO0PIN;
	pinset[1].pin_set = &FIO0SET;
	pinset[1].pin_clr = &FIO0CLR;
	pinset[1].pin_dir = &FIO0DIR;
	
	memcpy((void *)&pinset[2].pin_name, (const char *)&"P4_21", PIN_NAME_SIZE);
	pinset[2].pin_offset = 1<<21;
	pinset[2].pin_get = &FIO4PIN;
	pinset[2].pin_set = &FIO4SET;
	pinset[2].pin_clr = &FIO4CLR;
	pinset[2].pin_dir = &FIO4DIR;	
	
	memcpy((void *)&pinset[3].pin_name, (const char *)&"P4_20", PIN_NAME_SIZE);
	pinset[3].pin_offset = 1<<20;
	pinset[3].pin_get = &FIO4PIN;
	pinset[3].pin_set = &FIO4SET;
	pinset[3].pin_clr = &FIO4CLR;
	pinset[3].pin_dir = &FIO4DIR;	
	
	memcpy((void *)&pinset[4].pin_name, (const char *)&"P0_17", PIN_NAME_SIZE);
	pinset[4].pin_offset = 1<<17;
	pinset[4].pin_get = &FIO0PIN;
	pinset[4].pin_set = &FIO0SET;
	pinset[4].pin_clr = &FIO0CLR;
	pinset[4].pin_dir = &FIO0DIR;
	
	memcpy((void *)&pinset[5].pin_name, (const char *)&"P0_18", PIN_NAME_SIZE);
	pinset[5].pin_offset = 1<<18;
	pinset[5].pin_get = &FIO0PIN;
	pinset[5].pin_set = &FIO0SET;
	pinset[5].pin_clr = &FIO0CLR;
	pinset[5].pin_dir = &FIO0DIR;
	
	memcpy((void *)&pinset[6].pin_name, (const char *)&"P0_16", PIN_NAME_SIZE);
	pinset[6].pin_offset = 1<<16;
	pinset[6].pin_get = &FIO0PIN;
	pinset[6].pin_set = &FIO0SET;
	pinset[6].pin_clr = &FIO0CLR;
	pinset[6].pin_dir = &FIO0DIR;
	
	// conf the PULL PIN	
	memcpy((void *)&pull_pin.pin_name, (const char *)&"P0_15", PIN_NAME_SIZE);
	pull_pin.pin_offset = 1<<15;
	pull_pin.pin_get = &FIO0PIN;
	pull_pin.pin_set = &FIO0SET;
	pull_pin.pin_clr = &FIO0CLR;
	pull_pin.pin_dir = &FIO0DIR;
	
	// conf LED pin for indication, use the USB led P1_18
	memcpy((void *)&led_pin.pin_name, (const char *)&"P1_18", PIN_NAME_SIZE);
	led_pin.pin_offset = 1<<18;
	led_pin.pin_get = &FIO1PIN;
	led_pin.pin_set = &FIO1SET;
	led_pin.pin_clr = &FIO1CLR;
	led_pin.pin_dir = &FIO1DIR;
}

int pin_get(int pin)
{
	return ( (*pinset[pin].pin_get & pinset[pin].pin_offset) ? 1 : 0);
}

void pin_set_dir(int pin, int dir)
{
	if (dir == 0)
	{
		*pinset[pin].pin_dir &= ~(pinset[pin].pin_offset);
	}
	else
	{
		*pinset[pin].pin_dir |= pinset[pin].pin_offset;
	}
}

void pin_set(int pin)
{
	*pinset[pin].pin_set = pinset[pin].pin_offset;
}

void pin_clr(int pin)
{
	*pinset[pin].pin_clr = pinset[pin].pin_offset;
}

void pull_set_low(void)
{
	//int tempFIODIR;
	//tempFIODIR = *pull_pin.pin_dir;
	*pull_pin.pin_dir |= pull_pin.pin_offset;
	//tempFIODIR = *pull_pin.pin_dir;
	*pull_pin.pin_clr = pull_pin.pin_offset;
}

void pull_set_high(void)
{
	*pull_pin.pin_dir |= pull_pin.pin_offset;
	*pull_pin.pin_set = pull_pin.pin_offset;
}

void setallinput(void)
{
	int cntr;

	// make P02, P03, P015 GPIOs
	PINSEL0 &= ~(0x3<<4 | 0x3<<6 | 0x3<<30);
	// make P016, P017, P018 GPIOs
	PINSEL1 &= ~(0x3<<0 | 0x3<<2 | 0x3<<4);
    // make P420, P421 GPIOs
	PINSEL9 &= ~(0x3<<8 | 0x3<<10);
	
	// turn off the pull up/down
	PINMODE0 &= ~(0x3<<4 | 0x3<<6 | 0x3<<30);
	PINMODE1 &= ~(0x3<<0 | 0x3<<2 | 0x3<<4);
	PINMODE9 &= ~(0x3<<8 | 0x3<<10);
	PINMODE0 |= (0x2<<4 | 0x2<<6 | 0x2<<30);
	PINMODE1 |= (0x2<<0 | 0x2<<2 | 0x2<<4);
	PINMODE9 |= (0x2<<8 | 0x2<<10);
	
	
	for (cntr = 0; cntr < NUMBER_TESTED_PINS; cntr++)
	{
		pin_set_dir(cntr, 0);
	}
}

void ext_settling_delay(void)
{
	int i;
	for (i = EXT_SETTLING_DELAY; i; i--);
}

void TestExt(void)
{
	int cntr;
	int cntr1;
		
	PinsetInit();
	
	setallinput();
	pull_set_low();
	ext_settling_delay();
	
	for (cntr = 0; cntr < NUMBER_TESTED_PINS; cntr++)
	{
		if (1 == pin_get(cntr))
		{
			printf("\r\nPin %s to Vcc!!!!!!!!!!!!!!!!!", pinset[cntr].pin_name);
			return;
		}
	}
	
	setallinput();
	pull_set_high();
	ext_settling_delay();
	
	for (cntr = 0; cntr < NUMBER_TESTED_PINS; cntr++)
	{
		if (0 == pin_get(cntr))
		{
			printf("\r\nPin %s to GND!!!!!!!!!!!!!!!!!", pinset[cntr].pin_name);
			return;
		}
	}
	
	for (cntr = 0; cntr < NUMBER_TESTED_PINS; cntr++)
	{
		setallinput();
		pull_set_low();
		pin_set_dir(cntr, 1);
		pin_set(cntr);
		ext_settling_delay();
		
		for (cntr1 = 0; cntr1 < NUMBER_TESTED_PINS; cntr1++)
		{
			if (cntr1 == cntr)
			{
				if ( 0 == pin_get(cntr1))
				{
					printf("\r\nPin %s cannot go HIGH, probably short to %s!!!!!!!!!!!!", pinset[cntr1].pin_name, pull_pin.pin_name);
					return;
				}
			}
			else
			{
				if ( 1 == pin_get(cntr1))
				{
					printf("\r\nShort between pins %s and %s !!!!!!!!!!!", pinset[cntr1].pin_name, pinset[cntr].pin_name);
					return;
				}
			}
		}
		
	}
	
	printf("\r\nExtension test OK\r\n");
	
	/**led_pin.pin_dir |= led_pin.pin_offset;
	
	while (1)
	{
		*led_pin.pin_set = led_pin.pin_offset;
	
		*led_pin.pin_clr = led_pin.pin_offset;
	}*/
	
}
